import pytest
import numpy as np
from Animate.Movie import Movie
from Animate.Animation import Animation


def test_ratio_2():
    m = Movie(dt=1.0/14, height_ratio=2)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    a = Animation(m)
    assert a.gs.get_geometry() == (1, 1)
    assert a.gs.get_height_ratios() is None

    m = Movie(dt=1.0/14, height_ratio=2)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4))
    a = Animation(m)
    assert a.gs.get_geometry() == (2, 1)
    assert a.gs.get_height_ratios() == (2, 1)

    m = Movie(dt=1.0 / 14, height_ratio=2)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4))
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4), axis=1)
    a = Animation(m)
    assert a.gs.get_geometry() == (3, 1)
    assert a.gs.get_height_ratios() == (4, 1, 1)


def test_ratio_1():
    m = Movie(dt=1.0 / 14, height_ratio=1)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    a = Animation(m)
    assert a.gs.get_geometry() == (1, 1)
    assert a.gs.get_height_ratios() is None

    m = Movie(dt=1.0 / 14, height_ratio=1)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4))
    a = Animation(m)
    assert a.gs.get_geometry() == (2, 1)
    assert a.gs.get_height_ratios() == (1, 1)

    m = Movie(dt=1.0 / 14, height_ratio=1)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4))
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4), axis=1)
    a = Animation(m)
    assert a.gs.get_geometry() == (3, 1)
    assert a.gs.get_height_ratios() == (2, 1, 1)


def test_ratio_1p5():
    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    a = Animation(m)
    assert a.gs.get_geometry() == (1, 1)
    assert a.gs.get_height_ratios() is None

    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4))
    a = Animation(m)
    assert a.gs.get_geometry() == (2, 1)
    assert a.gs.get_height_ratios() == (1.5, 1)

    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4))
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4), axis=1)
    a = Animation(m)
    assert a.gs.get_geometry() == (3, 1)
    assert a.gs.get_height_ratios() == (3, 1, 1)


def test_n_axis():
    with pytest.raises(RuntimeError) as ex:
        m = Movie(dt=1.0 / 14, height_ratio=1.5)
        _ = Animation(m)
    assert 'At least one image is needed' in str(ex.value)

    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    a = Animation(m)
    assert a.n_axes == 0
    assert a.n_images == 1

    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_image(img, 'dark_img')
    a = Animation(m)
    assert a.n_axes == 0
    assert a.n_images == 2

    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    with pytest.raises(RuntimeError) as ex:
        _ = Animation(m)
    assert 'Axis 0 with no traces' in str(ex.value)

    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    m.add_axis('x', 'y')
    m.add_trace(np.arange(4))
    a = Animation(m)
    assert a.n_axes == 1
    assert a.n_images == 1


def test_x_data():
    m = Movie(dt=1.0 / 14, height_ratio=1.5)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img')
    a = Animation(m)
    assert np.allclose(a.x_data, np.arange(4) * 1.0 / 14)

    m = Movie(dt=0.5, height_ratio=1.5)
    img = np.arange(1000).reshape(40, 5, 5)
    m.add_image(img, 'dark_img')
    a = Animation(m)
    assert np.allclose(a.x_data, np.arange(40) * 0.5)



import pytest
import numpy as np
from Animate.Movie import Movie
from Animate.Animation import Animation


def test_set_ylim_image():
    m = Movie(dt=1.0/14)
    img = np.arange(100).reshape(4, 5, 5)
    m.add_image(img, 'dark_img', ylim_type='p_top', ylim_value=50)
    m_img = m.images[-1]
    assert m_img['ymin'] == 0
    assert np.isclose(m_img['ymax'], 49.5)

    m.add_image(img, 'dark_img', ylim_type='p_bottom', ylim_value=50)
    m_img = m.images[-1]
    assert np.isclose(m_img['ymin'], 49.5)
    assert m_img['ymax'] == 99

    m.add_image(img, 'dark_img', ylim_type='p_both', ylim_value=10)
    m_img = m.images[-1]
    assert np.isclose(m_img['ymin'], 9.9)
    assert np.isclose(m_img['ymax'], 89.1)

    m.add_image(img, 'dark_img', ylim_type='set', ylim_value=(2, 10))
    m_img = m.images[-1]
    assert m_img['ymin'] == 2
    assert m_img['ymax'] == 10

    with pytest.raises(RuntimeError) as ex:
        m.add_image(img, 'dark_img', ylim_type='set', ylim_value=10)
    assert 'ylim type set to set but len of ylim_value is not len 2' in str(ex.value)

    m.add_image(img, 'dark_img', ylim_type='same', ylim_value=len(m.images)-1)
    m_img = m.images[-1]
    assert m_img['ymin'] == 2
    assert m_img['ymax'] == 10

    with pytest.raises(RuntimeError) as ex:
        m.add_image(img, 'dark_img', ylim_type='same', ylim_value=len(m.images))
    assert 'Tried to have same y' in str(ex.value)

    with pytest.raises(RuntimeError) as ex:
        m.add_image(img, 'dark_img', ylim_type='error', ylim_value=len(m.images))
    assert "Expected 'p_top'" in str(ex.value)


def test_set_ylim_axis_top():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    data = np.arange(10)
    m.add_axis('test', 'test', ylim_type='p_top', ylim_value=50)
    m.add_trace(data)
    a = Animation(m)
    ax = a.trace_axes[-1]
    y_min, y_max = ax.get_ylim()
    assert np.isclose(y_min, 0)
    assert np.isclose(y_max, 4.5)


def test_set_ylim_axis_bottom():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    data = np.arange(10)
    m.add_axis('test', 'test', ylim_type='p_bottom', ylim_value=50)
    m.add_trace(data)
    a = Animation(m)
    ax = a.trace_axes[-1]
    y_min, y_max = ax.get_ylim()
    assert np.isclose(y_min, 4.5)
    assert np.isclose(y_max, 9)


def test_set_ylim_axis_both():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    data = np.arange(10)
    m.add_axis('test', 'test', ylim_type='p_both', ylim_value=10)
    m.add_trace(data)
    a = Animation(m)
    ax = a.trace_axes[-1]
    y_min, y_max = ax.get_ylim()
    assert np.isclose(y_min, 0.9)
    assert np.isclose(y_max, 8.1)


def test_set_ylim_axis_set_fail():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    data = np.arange(10)
    with pytest.raises(RuntimeError) as ex:
        m.add_axis('test', 'test', ylim_type='set', ylim_value=10)
        m.add_trace(data)
        a = Animation(m)
    assert 'ylim type set to set but len of ylim_value is not len 2' in str(ex.value)


def test_set_ylim_axis_set():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    data = np.arange(10)
    m.add_axis('test', 'test', ylim_type='set', ylim_value=(2, 10))
    m.add_trace(data)
    a = Animation(m)
    ax = a.trace_axes[-1]
    y_min, y_max = ax.get_ylim()
    assert np.isclose(y_min, 2)
    assert np.isclose(y_max, 10)


def test_set_ylim_axis_same_fail():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    with pytest.raises(RuntimeError) as ex:
        m.add_axis('test', 'test', ylim_type='same', ylim_value=1)
        m.add_trace(np.arange(10))
        _ = Animation(m)
    assert 'Tried to have same y limits ' in str(ex.value)


def test_set_ylim_axis_same():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    data = np.arange(10)
    m.add_axis('test', 'test', ylim_type='set', ylim_value=(2, 10))
    m.add_trace(data)
    m.add_axis('test', 'test', ylim_type='same', ylim_value=0)
    m.add_trace(data)
    m.add_trace(data, axis=1)
    a = Animation(m)
    ax = a.trace_axes[-1]
    y_min, y_max = ax.get_ylim()
    assert np.isclose(y_min, 2)
    assert np.isclose(y_max, 10)


def test_set_ylim_axis_two():
    m = Movie(dt=1.0 / 14)
    img = np.arange(250).reshape(10, 5, 5)
    m.add_image(img, 'dark_img')
    data = np.arange(10)
    m.add_axis('test', 'test', ylim_type='p_top', ylim_value=10)
    m.add_trace(data)
    m.add_trace(np.arange(10, 20))
    a = Animation(m)
    ax = a.trace_axes[-1]
    y_min, y_max = ax.get_ylim()
    assert np.isclose(y_min, 0)
    assert np.isclose(y_max, 17.1)
